pub struct Scheduler {
pub handoff: Handoff,
pub cadence: Cadence,
pub min_samples: u32,
pub sample_alignment: Option<Duration>,
}Expand description
A scheduler allows building a scheduling of spaceraft tracking for a set of ground stations.
Fields
handoff: HandoffHandoff strategy if two trackers see the vehicle at the same time
cadence: CadenceOn/off cadence of this scheduler
min_samples: u32Minimum number of samples for a valid arc, i.e. if there are less than this many samples during a pass, the strand is discarded.
sample_alignment: Option<Duration>Round the time of the samples to the provided duration. For example, if the vehicle is above the horizon at 01:02:03.456 and the alignment is set to 01 seconds, then this will cause the tracking to start at 01:02:03 as it is rounded to the nearest second.
Implementations
Sourceimpl Scheduler
impl Scheduler
Sourcepub fn builder() -> SchedulerBuilder<((), (), (), ())>
pub fn builder() -> SchedulerBuilder<((), (), (), ())>
Create a builder for building Scheduler.
On the builder, call .handoff(...)(optional), .cadence(...)(optional), .min_samples(...)(optional), .sample_alignment(...)(optional) to set the values of the fields.
Finally, call .build() to create the instance of Scheduler.