#[repr(u8)]pub enum Handoff {
Eager = 0,
Greedy = 1,
Overlap = 2,
}Expand description
Defines the handoff from a current ground station to the next one that is visible to prevent overlapping of measurements
Variants
Eager = 0
If a new station is in visibility of the spacecraft, the “Eager” station will immediately stop tracking and switch over (default)
Greedy = 1
If a new station is in visibility of the spacecraft, the “Greedy” station will continue to tracking until the vehicle is below its elevation mask
Overlap = 2
If a new station is in visibility of the spacecraft, the “Overlap” station will continue tracking, and so will the other one